@@ -8,11 +8,8 @@ import sys
 import os
 import re
 from collections import OrderedDict
-try:
-    from collections.abc import KeysView, ValuesView, ItemsView, Iterator
-except ImportError:
-    # pylint: disable=deprecated-class
-    from collections import KeysView, ValuesView, ItemsView, Iterator
+from collections.abc import KeysView, ValuesView, ItemsView, Iterator
+import unicodedata
 import pytest
 
 from ..utils.simplified_test_function import simplified_test_function
@@ -23,7 +20,7 @@ nocasedict = import_installed('nocasedict')
 from nocasedict import NocaseDict as _NocaseDict  # noqa: E402
 # pylint: enable=wrong-import-position, wrong-import-order, invalid-name
 
-PY2 = sys.version_info[0] == 2
+# pylint: disable=use-dict-literal
 
 # Controls whether the tests are run against a standard dict instead.
 TEST_AGAINST_DICT = os.getenv('TEST_DICT')
@@ -43,25 +40,18 @@ TESTDICT_IS_ORDERED = \
 TESTDICT_SUPPORTS_REVERSED = \
     not TEST_AGAINST_DICT or sys.version_info[0:2] >= (3, 8)
 
-# Indicates the dict being tested supports lt/gt comparison (between dicts)
-TESTDICT_SUPPORTS_COMPARISON = \
-    TEST_AGAINST_DICT and sys.version_info[0:2] == (2, 7)
-
 # Indicates the dict being tested issues UserWarning about not preserving order
 # of items in kwargs or in standard dict
 TESTDICT_WARNS_ORDER = \
     not TEST_AGAINST_DICT and sys.version_info[0:2] < (3, 7)
 
-# Indicates the dict supports the iter..() and view..() methods
-TESTDICT_SUPPORTS_ITER_VIEW = sys.version_info[0:2] == (2, 7)
-
 # Used as indicator not to pass an argument in the testcases.
 # Note this has nothing to do with the _OMITTED flag in _nocasedict.py and
 # could be a different value.
 _OMIT_ARG = object()
 
 
-class NonEquatable(object):
+class NonEquatable:
     # pylint: disable=too-few-public-methods
     """
     Class that raises TypeError when comparing its objects for equality.
@@ -366,7 +356,7 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError, None, True
     ),
     (
-        "Empty dict, with integer key (no lower / not found)",
+        "Empty dict, with integer key (no casefold / not found)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -375,7 +365,7 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError if TEST_AGAINST_DICT else AttributeError, None, True
     ),
     (
-        "Empty dict, with empty string key (not found)",
+        "Empty dict, with empty unicode string key (not found)",
         dict(
             obj=NocaseDict(),
             key='',
@@ -384,7 +374,16 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError, None, True
     ),
     (
-        "Empty dict, with non-empty key (not found)",
+        "Empty dict, with empty byte string key (not found)",
+        dict(
+            obj=NocaseDict(),
+            key=b'',
+            exp_value=None,
+        ),
+        KeyError, None, True
+    ),
+    (
+        "Empty dict, with non-empty unicode string key (not found)",
         dict(
             obj=NocaseDict(),
             key='Dog',
@@ -392,6 +391,15 @@ TESTCASES_NOCASEDICT_GETITEM = [
         ),
         KeyError, None, True
     ),
+    (
+        "Empty dict, with non-empty byte string key (not found)",
+        dict(
+            obj=NocaseDict(),
+            key=b'Dog',
+            exp_value=None,
+        ),
+        KeyError, None, True
+    ),
 
     # Non-empty NocaseDict
     (
@@ -404,7 +412,7 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError, None, True
     ),
     (
-        "Non-empty dict, with empty string key (not found)",
+        "Non-empty dict, with empty unicode string key (not found)",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             key='',
@@ -413,7 +421,17 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError, None, True
     ),
     (
-        "Non-empty dict, with non-empty non-existing key (not found)",
+        "Non-empty dict, with empty byte string key (not found)",
+        dict(
+            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
+            key=b'',
+            exp_value=None,
+        ),
+        KeyError, None, True
+    ),
+    (
+        "Non-empty dict, with non-empty non-existing unicode string key "
+        "(not found)",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             key='invalid',
@@ -422,7 +440,17 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError, None, True
     ),
     (
-        "Non-empty dict, with existing key in original case",
+        "Non-empty dict, with non-empty non-existing byte string key "
+        "(not found)",
+        dict(
+            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
+            key=b'invalid',
+            exp_value=None,
+        ),
+        KeyError, None, True
+    ),
+    (
+        "Non-empty dict, with existing unicode string key in original case",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             key='Dog',
@@ -431,7 +459,17 @@ TESTCASES_NOCASEDICT_GETITEM = [
         None, None, True
     ),
     (
-        "Non-empty dict, with existing key in non-original upper case",
+        "Non-empty dict, with existing byte string key in original case",
+        dict(
+            obj=NocaseDict([(b'Dog', 'Cat'), (b'Budgie', 'Fish')]),
+            key=b'Dog',
+            exp_value='Cat',
+        ),
+        None, None, True
+    ),
+    (
+        "Non-empty dict, with existing unicode string key in non-original "
+        "upper case",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             key='DOG',
@@ -440,7 +478,18 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError if TEST_AGAINST_DICT else None, None, True
     ),
     (
-        "Non-empty dict, with existing key in non-original lower case",
+        "Non-empty dict, with existing byte string key in non-original "
+        "upper case",
+        dict(
+            obj=NocaseDict([(b'Dog', 'Cat'), (b'Budgie', 'Fish')]),
+            key=b'DOG',
+            exp_value='Cat',
+        ),
+        KeyError if TEST_AGAINST_DICT else None, None, True
+    ),
+    (
+        "Non-empty dict, with existing unicode string key in non-original "
+        "lower case",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             key='dog',
@@ -449,7 +498,18 @@ TESTCASES_NOCASEDICT_GETITEM = [
         KeyError if TEST_AGAINST_DICT else None, None, True
     ),
     (
-        "Non-empty dict, with existing key in non-original mixed case",
+        "Non-empty dict, with existing byte string key in non-original "
+        "lower case",
+        dict(
+            obj=NocaseDict([(b'Dog', 'Cat'), (b'Budgie', 'Fish')]),
+            key=b'dog',
+            exp_value='Cat',
+        ),
+        KeyError if TEST_AGAINST_DICT else None, None, True
+    ),
+    (
+        "Non-empty dict, with existing unicode string key in non-original "
+        "mixed case",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             key='doG',
@@ -457,6 +517,34 @@ TESTCASES_NOCASEDICT_GETITEM = [
         ),
         KeyError if TEST_AGAINST_DICT else None, None, True
     ),
+    (
+        "Non-empty dict, with existing byte string key in non-original "
+        "mixed case",
+        dict(
+            obj=NocaseDict([(b'Dog', 'Cat'), (b'Budgie', 'Fish')]),
+            key=b'doG',
+            exp_value='Cat',
+        ),
+        KeyError if TEST_AGAINST_DICT else None, None, True
+    ),
+    (
+        "Non-empty unicode string dict, with same byte string key",
+        dict(
+            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
+            key=b'Dog',
+            exp_value='Cat',
+        ),
+        KeyError, None, True
+    ),
+    (
+        "Non-empty byte string dict, with same unicode string key",
+        dict(
+            obj=NocaseDict([(b'Dog', 'Cat'), (b'Budgie', 'Fish')]),
+            key='Dog',
+            exp_value='Cat',
+        ),
+        KeyError, None, True
+    ),
 ]
 
 
@@ -505,7 +593,7 @@ TESTCASES_NOCASEDICT_SETITEM = [
         None, None, True
     ),
     (
-        "Empty dict, with integer key (no lower / success)",
+        "Empty dict, with integer key (no casefold / success)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -646,7 +734,7 @@ TESTCASES_NOCASEDICT_DELITEM = [
         KeyError, None, True
     ),
     (
-        "Empty dict, with integer key (no lower / not found)",
+        "Empty dict, with integer key (no casefold / not found)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -830,7 +918,7 @@ TESTCASES_NOCASEDICT_CONTAINS = [
         None, None, True
     ),
     (
-        "Empty dict, with integer key (no lower / success)",
+        "Empty dict, with integer key (no casefold / success)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -944,148 +1032,6 @@ def test_NocaseDict_contains(testcase, obj, key, exp_result):
         "Unexpected result at key {k!r}".format(k=key)
 
 
-TESTCASES_NOCASEDICT_HAS_KEY = [
-
-    # Testcases for NocaseDict.has_key()
-
-    # Each list item is a testcase tuple with these items:
-    # * desc: Short testcase description.
-    # * kwargs: Keyword arguments for the test function:
-    #   * obj: NocaseDict object to be used for the test.
-    #   * key: Key to be used for the test.
-    #   * exp_result: Expected result (bool).
-    # * exp_exc_types: Expected exception type(s), or None.
-    # * exp_warn_types: Expected warning type(s), or None.
-    # * condition: Boolean condition for testcase to run, or 'pdb' for debugger
-
-    # Empty NocaseDict
-    (
-        "Empty dict, with None key",
-        dict(
-            obj=NocaseDict(),
-            key=None,
-            exp_result=False,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Empty dict, with integer key (no lower / success)",
-        dict(
-            obj=NocaseDict(),
-            key=1234,
-            exp_result=False if TEST_AGAINST_DICT else None,
-        ),
-        AttributeError if not PY2 \
-        else None if TEST_AGAINST_DICT \
-        else AttributeError,
-        None, True
-    ),
-    (
-        "Empty dict, with empty string key (not found)",
-        dict(
-            obj=NocaseDict(),
-            key='',
-            exp_result=False,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Empty dict, with non-empty key (not found)",
-        dict(
-            obj=NocaseDict(),
-            key='Dog',
-            exp_result=False,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-
-    # Non-empty NocaseDict
-    (
-        "Non-empty dict, with non-existing None key",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key=None,
-            exp_result=False,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Non-empty dict, with empty non-existing string key (not found)",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key='',
-            exp_result=False,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Non-empty dict, with non-empty non-existing key (not found)",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key='invalid',
-            exp_result=False,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Non-empty dict, with existing key in original case",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key='Dog',
-            exp_result=True,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Non-empty dict, with existing key in non-original upper case",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key='DOG',
-            exp_result=not TEST_AGAINST_DICT,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Non-empty dict, with existing key in non-original lower case",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key='dog',
-            exp_result=not TEST_AGAINST_DICT,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-    (
-        "Non-empty dict, with existing key in non-original mixed case",
-        dict(
-            obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
-            key='doG',
-            exp_result=not TEST_AGAINST_DICT,
-        ),
-        AttributeError if not PY2 else None, None, True
-    ),
-]
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_HAS_KEY)
-@simplified_test_function
-def test_NocaseDict_has_key(testcase, obj, key, exp_result):
-    """
-    Test function for NocaseDict.has_key()
-    """
-
-    # The code to be tested
-    act_result = obj.has_key(key)  # noqa: W601
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    assert act_result == exp_result, \
-        "Unexpected result at key {k!r}".format(k=key)
-
-
 TESTCASES_NOCASEDICT_FROMKEYS = [
 
     # Testcases for NocaseDict.fromkeys()
@@ -1323,7 +1269,7 @@ def test_NocaseDict_reversed(testcase, obj, exp_keys):
     # The reason we verify that an iterator is returned is that
     # NocaseDict.__reversed__() delegates to keys() which returns a list in
     # Python 2, so this verifies that reversed() still turns this into an
-    # iterator.
+    # iterator. Maybe no longer needed on Python 3, but it does not hurt.
     assert isinstance(act_keys_iter, Iterator)
 
     act_keys = list(act_keys_iter)
@@ -1357,7 +1303,7 @@ TESTCASES_NOCASEDICT_GET = [
         None, None, True
     ),
     (
-        "Empty dict, with integer key (no lower / success)",
+        "Empty dict, with integer key (no casefold / success)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -1557,7 +1503,7 @@ TESTCASES_NOCASEDICT_POP = [
     ),
     (
         "Empty dict, with integer key and default omitted "
-        "(no lower / dict empty)",
+        "(no casefold / dict empty)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -1776,7 +1722,7 @@ TESTCASES_NOCASEDICT_SETDEFAULT = [
         None, None, True
     ),
     (
-        "Empty dict, with integer key (no lower / success)",
+        "Empty dict, with integer key (no casefold / success)",
         dict(
             obj=NocaseDict(),
             key=1234,
@@ -2000,234 +1946,6 @@ def test_NocaseDict_keys(testcase, obj, exp_items):
     # Test that second iteration is possible
     act_keys_list2 = list(act_keys)
 
-    if not PY2:
-
-        # Test __contained__() of the returned view
-        for key in act_keys_list:
-            assert key in act_keys
-
-        # Test __repr__() of the returned view
-        assert re.match(r'^dict_keys\(.*\)$', repr(act_keys))
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    if PY2:
-        assert isinstance(act_keys, list)
-    else:
-        assert isinstance(act_keys, KeysView)
-
-    exp_keys = [item[0] for item in exp_items]
-    if TESTDICT_IS_ORDERED:
-        assert act_keys_list == exp_keys
-        assert act_keys_list2 == exp_keys
-    else:
-        assert sorted(act_keys_list) == sorted(exp_keys)
-        assert sorted(act_keys_list2) == sorted(exp_keys)
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_ITEMS)
-@simplified_test_function
-def test_NocaseDict_values(testcase, obj, exp_items):
-    """
-    Test function for NocaseDict.values()
-    """
-
-    # The code to be tested
-    act_values = obj.values()
-
-    # Also test iterating through the result
-    act_values_list = list(act_values)
-
-    # Test that second iteration is possible
-    act_values_list2 = list(act_values)
-
-    if not PY2:
-
-        # Test __contained__() of the returned view
-        for value in act_values_list:
-            assert value in act_values
-
-        # Test __repr__() of the returned view
-        assert re.match(r'^dict_values\(.*\)$', repr(act_values))
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    if PY2:
-        assert isinstance(act_values, list)
-    else:
-        assert isinstance(act_values, ValuesView)
-
-    exp_values = [item[1] for item in exp_items]
-    if TESTDICT_IS_ORDERED:
-        assert act_values_list == exp_values
-        assert act_values_list2 == exp_values
-    else:
-        assert sorted(act_values_list) == sorted(exp_values)
-        assert sorted(act_values_list2) == sorted(exp_values)
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_ITEMS)
-@simplified_test_function
-def test_NocaseDict_items(testcase, obj, exp_items):
-    """
-    Test function for NocaseDict.items()
-    """
-
-    # The code to be tested
-    act_items = obj.items()
-
-    # Also test iterating through the result
-    act_items_list = list(act_items)
-
-    # Test that second iteration is possible
-    act_items_list2 = list(act_items)
-
-    if not PY2:
-
-        # Test __contained__() of the returned view
-        for item in act_items_list:
-            assert item in act_items
-
-        # Test __repr__() of the returned view
-        assert re.match(r'^dict_items\(.*\)$', repr(act_items))
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    if PY2:
-        assert isinstance(act_items, list)
-    else:
-        assert isinstance(act_items, ItemsView)
-
-    if TESTDICT_IS_ORDERED:
-        assert act_items_list == exp_items
-        assert act_items_list2 == exp_items
-    else:
-        assert sorted(act_items_list) == sorted(exp_items)
-        assert sorted(act_items_list2) == sorted(exp_items)
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_ITEMS)
-@simplified_test_function
-def test_NocaseDict_iterkeys(testcase, obj, exp_items):
-    """
-    Test function for NocaseDict.iterkeys()
-    """
-
-    if not TESTDICT_SUPPORTS_ITER_VIEW:
-        pytest.skip("Test dictionary does not support iterkeys() method")
-
-    assert PY2
-
-    # The code to be tested
-    act_keys = []
-    for key in obj.iterkeys():
-        act_keys.append(key)
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    exp_keys = [item[0] for item in exp_items]
-    if TESTDICT_IS_ORDERED:
-        assert act_keys == exp_keys
-    else:
-        assert sorted(act_keys) == sorted(exp_keys)
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_ITEMS)
-@simplified_test_function
-def test_NocaseDict_itervalues(testcase, obj, exp_items):
-    """
-    Test function for NocaseDict.itervalues()
-    """
-
-    if not TESTDICT_SUPPORTS_ITER_VIEW:
-        pytest.skip("Test dictionary does not support itervalues() method")
-
-    assert PY2
-
-    # The code to be tested
-    act_values = []
-    for value in obj.itervalues():
-        act_values.append(value)
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    exp_values = [item[1] for item in exp_items]
-    if TESTDICT_IS_ORDERED:
-        assert act_values == exp_values
-    else:
-        assert sorted(act_values) == sorted(exp_values)
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_ITEMS)
-@simplified_test_function
-def test_NocaseDict_iteritems(testcase, obj, exp_items):
-    """
-    Test function for NocaseDict.iteritemss()
-    """
-
-    if not TESTDICT_SUPPORTS_ITER_VIEW:
-        pytest.skip("Test dictionary does not support iteritems() method")
-
-    assert PY2
-
-    # The code to be tested
-    act_items = []
-    for item in obj.iteritems():
-        act_items.append(item)
-
-    # Ensure that exceptions raised in the remainder of this function
-    # are not mistaken as expected exceptions
-    assert testcase.exp_exc_types is None
-
-    if TESTDICT_IS_ORDERED:
-        assert act_items == exp_items
-    else:
-        assert sorted(act_items) == sorted(exp_items)
-
-
-@pytest.mark.parametrize(
-    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
-    TESTCASES_NOCASEDICT_ITEMS)
-@simplified_test_function
-def test_NocaseDict_viewkeys(testcase, obj, exp_items):
-    """
-    Test function for NocaseDict.viewkeys() (PY2 only)
-    """
-
-    if not TESTDICT_SUPPORTS_ITER_VIEW:
-        pytest.skip("Test dictionary does support viewkeys() method")
-
-    assert PY2
-
-    # The code to be tested
-    act_keys = obj.viewkeys()
-
-    # Also test iterating through the result
-    act_keys_list = list(act_keys)
-
-    # Test that second iteration is possible
-    act_keys_list2 = list(act_keys)
-
     # Test __contained__() of the returned view
     for key in act_keys_list:
         assert key in act_keys
@@ -2254,18 +1972,13 @@ def test_NocaseDict_viewkeys(testcase, obj, exp_items):
     "desc, kwargs, exp_exc_types, exp_warn_types, condition",
     TESTCASES_NOCASEDICT_ITEMS)
 @simplified_test_function
-def test_NocaseDict_viewvalues(testcase, obj, exp_items):
+def test_NocaseDict_values(testcase, obj, exp_items):
     """
-    Test function for NocaseDict.viewvalues()
+    Test function for NocaseDict.values()
     """
 
-    if not TESTDICT_SUPPORTS_ITER_VIEW:
-        pytest.skip("Test dictionary does not support viewvalues() method")
-
-    assert PY2
-
     # The code to be tested
-    act_values = obj.viewvalues()
+    act_values = obj.values()
 
     # Also test iterating through the result
     act_values_list = list(act_values)
@@ -2299,18 +2012,13 @@ def test_NocaseDict_viewvalues(testcase, obj, exp_items):
     "desc, kwargs, exp_exc_types, exp_warn_types, condition",
     TESTCASES_NOCASEDICT_ITEMS)
 @simplified_test_function
-def test_NocaseDict_viewitems(testcase, obj, exp_items):
+def test_NocaseDict_items(testcase, obj, exp_items):
     """
-    Test function for NocaseDict.viewitems()
+    Test function for NocaseDict.items()
     """
 
-    if not TESTDICT_SUPPORTS_ITER_VIEW:
-        pytest.skip("Test dictionary does not support viewitems() method")
-
-    assert PY2
-
     # The code to be tested
-    act_items = obj.viewitems()
+    act_items = obj.items()
 
     # Also test iterating through the result
     act_items_list = list(act_items)
@@ -2456,7 +2164,7 @@ TESTCASES_NOCASEDICT_UPDATE = [
         TypeError, None, True
     ),
     (
-        "Empty dict, with integer key in update args (no lower / success)",
+        "Empty dict, with integer key in update args (no casefold / success)",
         dict(
             obj=NocaseDict(),
             args=[[(1234, 'Invalid')]],
@@ -2572,7 +2280,8 @@ TESTCASES_NOCASEDICT_UPDATE = [
 
     # Non-empty NocaseDict, insert new value
     (
-        "Non-empty dict, with integer key in update args (no lower / success)",
+        "Non-empty dict, with integer key in update args "
+        "(no casefold / success)",
         dict(
             obj=NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')]),
             args=[[(1234, 'Invalid')]],
@@ -3171,7 +2880,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Empty dicts with >=",
@@ -3181,7 +2890,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>=',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Empty dicts with <",
@@ -3191,7 +2900,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Empty dicts with <=",
@@ -3201,7 +2910,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<=',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
 
     # Equal dicts
@@ -3213,7 +2922,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Equal dicts with >=",
@@ -3223,7 +2932,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>=',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Equal dicts with <",
@@ -3233,7 +2942,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Equal dicts with <=",
@@ -3243,7 +2952,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<=',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
 
     # Dicts that compare less (obj1 < obj2)
@@ -3255,7 +2964,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Less-comparing dicts with >=",
@@ -3265,7 +2974,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>=',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Less-comparing dicts with <",
@@ -3275,7 +2984,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Less-comparing dicts with <=",
@@ -3285,7 +2994,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<=',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
 
     # Dicts that compare greater (obj1 > obj2)
@@ -3297,7 +3006,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Greater-comparing dicts with >=",
@@ -3307,7 +3016,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='>=',
             exp_result=True,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Greater-comparing dicts with <",
@@ -3317,7 +3026,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
     (
         "Greater-comparing dicts with <=",
@@ -3327,7 +3036,7 @@ TESTCASES_NOCASEDICT_ORDERING = [
             op='<=',
             exp_result=False,
         ),
-        None if TESTDICT_SUPPORTS_COMPARISON else TypeError, None, True
+        TypeError, None, True
     ),
 
     # Note: More subtle cases of less- or greater-comparing dicts are not
@@ -3382,3 +3091,31 @@ def test_unnamed_keys():
     del dic[None]
     assert None not in dic
     assert not dic
+
+
+def test_casefold_override():
+    """
+    Test function for overriding the casefold method.
+    """
+
+    if TEST_AGAINST_DICT:
+        pytest.skip("The override test does not support testing with dict")
+
+    class MyNocaseDict(NocaseDict):
+        "Test class that overrides the casefold method"
+
+        @staticmethod
+        def __casefold__(key):
+            return unicodedata.normalize('NFKD', key).casefold()
+
+    dic = MyNocaseDict()
+
+    exp_value = 'foo'
+
+    # Add item with combined Unicode character
+    dic["\u00C7"] = exp_value
+
+    # Look up item with combination sequence
+    act_value = dic["c\u0327"]
+
+    assert act_value == exp_value
